@@ -17,9 +17,9 @@
     Boston, MA 02110-1301, USA.
 */
 
-#include <qtest.h>
-
 #include "fileundomanagertest.h"
+
+#include <qtest.h>
 #include <QSignalSpy>
 #include <QDateTime>
 #include <QDir>
@@ -34,6 +34,7 @@
 #include <kio/paste.h>
 #include <kio/pastejob.h>
 #include <kio/restorejob.h>
+#include <kio/batchrenamejob.h>
 #include <kioglobal_p.h>
 #include <kprotocolinfo.h>
 #include <kurlmimedata.h>
@@ -51,7 +52,6 @@
 #include <sys/time.h>
 #endif
 
-
 #include <QClipboard>
 #include <QApplication>
 #include <QMimeData>
@@ -173,20 +173,17 @@ public:
     {
         setShowProgressInfo(false);
     }
-    void jobError(KIO::Job *job) Q_DECL_OVERRIDE
-    {
+    void jobError(KIO::Job *job) override {
         qFatal("%s", qPrintable(job->errorString()));
     }
-    bool copiedFileWasModified(const QUrl &src, const QUrl &dest, const QDateTime &srcTime, const QDateTime &destTime) Q_DECL_OVERRIDE
-    {
+    bool copiedFileWasModified(const QUrl &src, const QUrl &dest, const QDateTime &srcTime, const QDateTime &destTime) override {
         Q_UNUSED(src);
         m_dest = dest;
         Q_UNUSED(srcTime);
         Q_UNUSED(destTime);
         return true;
     }
-    bool confirmDeletion(const QList<QUrl> &files) Q_DECL_OVERRIDE
-    {
+    bool confirmDeletion(const QList<QUrl> &files) override {
         m_files = files;
         return m_nextReplyToConfirmDeletion;
     }
@@ -217,7 +214,7 @@ void FileUndoManagerTest::initTestCase()
 {
     qDebug("initTestCase");
 
-    QStandardPaths::enableTestMode(true);
+    QStandardPaths::setTestModeEnabled(true);
 
     // Get kio_trash to share our environment so that it writes trashrc to the right kdehome
     qputenv("KDE_FORK_SLAVES", "yes");
@@ -272,7 +269,7 @@ void FileUndoManagerTest::testCopyFiles()
     QList<QUrl> lst = sourceList();
     const QUrl d = QUrl::fromLocalFile(destdir);
     KIO::CopyJob *job = KIO::copy(lst, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordCopyJob(job);
 
     QSignalSpy spyUndoAvailable(FileUndoManager::self(), SIGNAL(undoAvailable(bool)));
@@ -328,7 +325,7 @@ void FileUndoManagerTest::testMoveFiles()
     QList<QUrl> lst = sourceList();
     const QUrl d = QUrl::fromLocalFile(destdir);
     KIO::CopyJob *job = KIO::move(lst, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordCopyJob(job);
 
     bool ok = job->exec();
@@ -371,7 +368,7 @@ void FileUndoManagerTest::testCopyDirectory()
     QList<QUrl> lst; lst << QUrl::fromLocalFile(srcSubDir());
     const QUrl d = QUrl::fromLocalFile(destdir);
     KIO::CopyJob *job = KIO::copy(lst, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordCopyJob(job);
 
     bool ok = job->exec();
@@ -392,7 +389,7 @@ void FileUndoManagerTest::testMoveDirectory()
     QList<QUrl> lst; lst << QUrl::fromLocalFile(srcSubDir());
     const QUrl d = QUrl::fromLocalFile(destdir);
     KIO::CopyJob *job = KIO::move(lst, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordCopyJob(job);
 
     bool ok = job->exec();
@@ -416,7 +413,7 @@ void FileUndoManagerTest::testRenameFile()
     QSignalSpy spyUndoAvailable(FileUndoManager::self(), SIGNAL(undoAvailable(bool)));
     QVERIFY(spyUndoAvailable.isValid());
     KIO::Job *job = KIO::moveAs(oldUrl, newUrl, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordJob(FileUndoManager::Rename, lst, newUrl, job);
 
     bool ok = job->exec();
@@ -439,7 +436,7 @@ void FileUndoManagerTest::testRenameDir()
     QList<QUrl> lst;
     lst.append(oldUrl);
     KIO::Job *job = KIO::moveAs(oldUrl, newUrl, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordJob(FileUndoManager::Rename, lst, newUrl, job);
 
     bool ok = job->exec();
@@ -454,6 +451,34 @@ void FileUndoManagerTest::testRenameDir()
     QVERIFY(!QFileInfo(newUrl.toLocalFile()).isDir());
 }
 
+void FileUndoManagerTest::testCreateSymlink()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows for lack of proper symlink support");
+#endif
+    const QUrl link = QUrl::fromLocalFile(homeTmpDir() + "newlink");
+    const QString path = link.toLocalFile();
+    QVERIFY(!QFile::exists(path));
+
+    const QUrl target = QUrl::fromLocalFile(homeTmpDir() + "linktarget");
+    const QString targetPath = target.toLocalFile();
+    createTestFile(targetPath, "Link's Target");
+    QVERIFY(QFile::exists(targetPath));
+
+    KIO::CopyJob *job = KIO::link(target, link);
+    job->setUiDelegate(nullptr);
+    FileUndoManager::self()->recordCopyJob(job);
+    bool ok = job->exec();
+    QVERIFY(ok);
+    QVERIFY(QFile::exists(path));
+    QVERIFY(QFileInfo(path).isSymLink());
+
+    // For undoing symlinks no confirmation is required. We delete it straight away.
+    doUndo();
+
+    QVERIFY(!QFile::exists(path));
+}
+
 void FileUndoManagerTest::testCreateDir()
 {
     const QUrl url = QUrl::fromLocalFile(srcSubDir() + ".mkdir");
@@ -461,7 +486,7 @@ void FileUndoManagerTest::testCreateDir()
     QVERIFY(!QFile::exists(path));
 
     KIO::SimpleJob *job = KIO::mkdir(url);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordJob(FileUndoManager::Mkdir, QList<QUrl>(), url, job);
     bool ok = job->exec();
     QVERIFY(ok);
@@ -491,7 +516,7 @@ void FileUndoManagerTest::testMkpath()
     const QUrl url = QUrl::fromLocalFile(path);
 
     KIO::Job *job = KIO::mkpath(url);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordJob(FileUndoManager::Mkpath, QList<QUrl>(), url, job);
     QVERIFY(job->exec());
     QVERIFY(QFileInfo(path).isDir());
@@ -518,7 +543,7 @@ void FileUndoManagerTest::testTrashFiles()
     QList<QUrl> lst = sourceList();
     lst.append(QUrl::fromLocalFile(srcSubDir()));
     KIO::Job *job = KIO::trash(lst, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordJob(FileUndoManager::Trash, lst, QUrl(QStringLiteral("trash:/")), job);
 
     bool ok = job->exec();
@@ -559,12 +584,12 @@ void FileUndoManagerTest::testRestoreTrashedFiles()
     QList<QUrl> lst = sourceList();
     lst.append(QUrl::fromLocalFile(srcSubDir()));
     KIO::Job *job = KIO::trash(lst, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     QVERIFY(job->exec());
 
     const QMap<QString, QString> metaData = job->metaData();
     QList<QUrl> trashUrls;
-    foreach(const QUrl &src, lst) {
+    foreach (const QUrl &src, lst) {
         QMap<QString, QString>::ConstIterator it = metaData.find("trashURL-" + src.path());
         QVERIFY(it != metaData.constEnd());
         trashUrls.append(QUrl(it.value()));
@@ -574,7 +599,7 @@ void FileUndoManagerTest::testRestoreTrashedFiles()
 
     // Restore from trash
     KIO::RestoreJob *restoreJob = KIO::restoreFromTrash(trashUrls, KIO::HideProgressInfo);
-    restoreJob->setUiDelegate(0);
+    restoreJob->setUiDelegate(nullptr);
     QVERIFY(restoreJob->exec());
 
     QVERIFY(QFile::exists(srcFile()));
@@ -593,7 +618,7 @@ static void setTimeStamp(const QString &path)
     // Put timestamp in the past so that we can check that the
     // copy actually preserves it.
     struct timeval tp;
-    gettimeofday(&tp, 0);
+    gettimeofday(&tp, nullptr);
     struct utimbuf utbuf;
     utbuf.actime = tp.tv_sec + 30; // 30 seconds in the future
     utbuf.modtime = tp.tv_sec + 60; // 60 second in the future
@@ -609,7 +634,7 @@ void FileUndoManagerTest::testModifyFileBeforeUndo()
     QList<QUrl> lst; lst << QUrl::fromLocalFile(srcSubDir());
     const QUrl d = QUrl::fromLocalFile(destdir);
     KIO::CopyJob *job = KIO::copy(lst, d, KIO::HideProgressInfo);
-    job->setUiDelegate(0);
+    job->setUiDelegate(nullptr);
     FileUndoManager::self()->recordCopyJob(job);
 
     bool ok = job->exec();
@@ -660,5 +685,73 @@ void FileUndoManagerTest::testPasteClipboardUndo()
     QCOMPARE(clipboardUrls, urls);
 }
 
+void FileUndoManagerTest::testBatchRename()
+{
+    auto createUrl = [](const QString &path) -> QUrl {
+        return QUrl::fromLocalFile(homeTmpDir() + path);
+    };
+
+    QList<QUrl> srcList;
+    srcList << createUrl("textfile.txt") << createUrl("mediafile.mkv") << createUrl("sourcefile.cpp");
+
+    createTestFile(srcList.at(0).path(), "foo");
+    createTestFile(srcList.at(1).path(), "foo");
+    createTestFile(srcList.at(2).path(), "foo");
+
+    KIO::Job *job = KIO::batchRename(srcList, QLatin1String("newfile###"), 1, QLatin1Char('#'));
+    job->setUiDelegate(nullptr);
+    FileUndoManager::self()->recordJob(FileUndoManager::BatchRename, srcList, QUrl(), job);
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+
+    QVERIFY(QFile::exists(createUrl("newfile001.txt").path()));
+    QVERIFY(QFile::exists(createUrl("newfile002.mkv").path()));
+    QVERIFY(QFile::exists(createUrl("newfile003.cpp").path()));
+    QVERIFY(!QFile::exists(srcList.at(0).path()));
+    QVERIFY(!QFile::exists(srcList.at(1).path()));
+    QVERIFY(!QFile::exists(srcList.at(2).path()));
+
+    doUndo();
+
+    QVERIFY(!QFile::exists(createUrl("newfile###.txt").path()));
+    QVERIFY(!QFile::exists(createUrl("newfile###.mkv").path()));
+    QVERIFY(!QFile::exists(createUrl("newfile###.cpp").path()));
+    QVERIFY(QFile::exists(srcList.at(0).path()));
+    QVERIFY(QFile::exists(srcList.at(1).path()));
+    QVERIFY(QFile::exists(srcList.at(2).path()));
+}
+
+void FileUndoManagerTest::testUndoCopyOfDeletedFile()
+{
+    const QUrl source = QUrl::fromLocalFile(homeTmpDir() + QLatin1String("source.txt"));
+    const QUrl dest = QUrl::fromLocalFile(homeTmpDir() + QLatin1String("copy.txt"));
+
+    createTestFile(source.toLocalFile(), "foo");
+    QVERIFY(QFileInfo::exists(source.toLocalFile()));
+
+    {
+        auto copyJob = KIO::copy(source, dest, KIO::HideProgressInfo);
+        copyJob->setUiDelegate(nullptr);
+        FileUndoManager::self()->recordCopyJob(copyJob);
+        QVERIFY2(copyJob->exec(), qPrintable(copyJob->errorString()));
+        QVERIFY(QFileInfo::exists(dest.toLocalFile()));
+    }
+
+    {
+        auto deleteJob = KIO::del(dest, KIO::HideProgressInfo);
+        deleteJob->setUiDelegate(nullptr);
+        QVERIFY2(deleteJob->exec(), qPrintable(deleteJob->errorString()));
+        QVERIFY(!QFileInfo::exists(dest.toLocalFile()));
+    }
+
+    QVERIFY(FileUndoManager::self()->undoAvailable());
+    QSignalSpy spyUndoAvailable(FileUndoManager::self(), static_cast<void (FileUndoManager::*)(bool)>(&FileUndoManager::undoAvailable));
+    QVERIFY(spyUndoAvailable.isValid());
+    // We can't use doUndo() because there is no UndoJob, so the nested event loop would never quit.
+    FileUndoManager::self()->undo();
+    QCOMPARE(spyUndoAvailable.count(), 1);
+    QVERIFY(!spyUndoAvailable.at(0).at(0).toBool());
+    QVERIFY(!FileUndoManager::self()->undoAvailable());
+}
+
 // TODO: add test (and fix bug) for  DND of remote urls / "Link here" (creates .desktop files) // Undo (doesn't do anything)
 // TODO: add test for interrupting a moving operation and then using Undo - bug:91579
