@@ -178,8 +178,8 @@ void KFileItemTest::testRootDirectory()
     const QString rootPath = QDir::rootPath();
     QUrl url = QUrl::fromLocalFile(rootPath);
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("."));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     KFileItem fileItem(entry, url);
     QCOMPARE(fileItem.text(), QStringLiteral("."));
     QVERIFY(fileItem.isLocalFile());
@@ -294,12 +294,48 @@ void KFileItemTest::testCmp()
     QVERIFY(fileItem.cmp(fileItem2));
 }
 
+void KFileItemTest::testCmpByUrl()
+{
+    const QUrl nulUrl;
+    const QUrl url  = QUrl::fromLocalFile(QStringLiteral("1foo"));
+    const QUrl url2 = QUrl::fromLocalFile(QStringLiteral("fo1"));
+    const QUrl url3 = QUrl::fromLocalFile(QStringLiteral("foo"));
+    KFileItem nulFileItem;
+    KFileItem nulFileItem2(nulUrl);
+    KFileItem fileItem(url);
+    KFileItem fileItem2(url2);
+    KFileItem fileItem3(url3);
+
+    // an invalid KFileItem is considered equal to any other invalid KFileItem or invalid QUrl.
+    QVERIFY(!(nulFileItem < nulFileItem));
+    QVERIFY(!(nulFileItem < nulFileItem2));
+    QVERIFY(!(nulFileItem2 < nulFileItem));
+    QVERIFY(!(nulFileItem < nulUrl));
+    // an invalid KFileItem is considered less than any valid KFileItem.
+    QVERIFY(nulFileItem < fileItem);
+    // a valid KFileItem is not less than an invalid KFileItem or invalid QUrl
+    QVERIFY(!(fileItem < nulUrl));
+    QVERIFY(!(fileItem < nulFileItem));
+    QVERIFY(!(fileItem < nulFileItem2));
+
+    QVERIFY(fileItem < fileItem2);
+    QVERIFY(fileItem < url2);
+    QVERIFY(!(fileItem2 < fileItem));
+    QVERIFY(fileItem2 < fileItem3);
+    QVERIFY(fileItem < url3);
+    QVERIFY(!(fileItem3 < fileItem2));
+    QVERIFY(!(fileItem3 < fileItem));
+    // Must be false as they are considered equal
+    QVERIFY(!(fileItem < fileItem));
+    QVERIFY(!(fileItem < url));
+}
+
 void KFileItemTest::testRename()
 {
     KIO::UDSEntry entry;
     const QString origName = QStringLiteral("foo");
-    entry.insert(KIO::UDSEntry::UDS_NAME, origName);
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, origName);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     KFileItem fileItem(entry, QUrl::fromLocalFile(QStringLiteral("/dir/foo")));
     QCOMPARE(fileItem.name(), origName);
     QCOMPARE(fileItem.text(), origName);
@@ -345,8 +381,16 @@ void KFileItemTest::testRefresh()
 #endif
     // Qt 5.8 adds milliseconds (but UDSEntry has no support for that)
     lastModified = dirInfo.lastModified();
+    // Truncate away the milliseconds...
     lastModified = lastModified.addMSecs(-lastModified.time().msec());
-    QCOMPARE(fileItem.time(KFileItem::ModificationTime), lastModified);
+    // ...but it looks like the kernel rounds up when the msecs are .998 or .999,
+    // so add a bit of tolerance
+    auto expectedLastModified = lastModified;
+    if (fileItem.time(KFileItem::ModificationTime) != lastModified &&
+            fileItem.time(KFileItem::ModificationTime) == lastModified.addSecs(1)) {
+        expectedLastModified = expectedLastModified.addSecs(1);
+    }
+    QCOMPARE(fileItem.time(KFileItem::ModificationTime), expectedLastModified);
     fileItem.refresh();
     QVERIFY(fileItem.isFile());
     QVERIFY(!fileItem.isLink());
@@ -354,7 +398,7 @@ void KFileItemTest::testRefresh()
 #ifndef Q_OS_WIN
     QCOMPARE(fileItem.user(), KUser().loginName());
 #endif
-    QCOMPARE(fileItem.time(KFileItem::ModificationTime), lastModified);
+    QCOMPARE(fileItem.time(KFileItem::ModificationTime), expectedLastModified);
 
     // Refresh on a symlink to a file
     const QString symlink = tempDir.path() + "/asymlink";
@@ -362,7 +406,7 @@ void KFileItemTest::testRefresh()
     QDateTime symlinkTime = QDateTime::currentDateTime().addSecs(-20);
     // we currently lose milliseconds....
     symlinkTime = symlinkTime.addMSecs(-symlinkTime.time().msec());
-    setTimeStamp(symlink, symlinkTime); // differenciate link time and source file time
+    setTimeStamp(symlink, symlinkTime); // differentiate link time and source file time
     const QUrl symlinkUrl = QUrl::fromLocalFile(symlink);
     KFileItem symlinkItem(symlinkUrl);
     QVERIFY(symlinkItem.isFile());
@@ -448,21 +492,22 @@ void KFileItemTest::testListProperties_data()
     QTest::addColumn<bool>("expectedDeleting");
     QTest::addColumn<bool>("expectedIsLocal");
     QTest::addColumn<bool>("expectedIsDirectory");
+    QTest::addColumn<bool>("expectedIsFile");
     QTest::addColumn<QString>("expectedMimeType");
     QTest::addColumn<QString>("expectedMimeGroup");
 
-    QTest::newRow("one file") << "f" << true << true << true << false << "text/plain" << "text";
-    QTest::newRow("one dir") << "d" << true << true << true << true << "inode/directory" << "inode";
-    QTest::newRow("root dir") << "/" << true << false << true << true << "inode/directory" << "inode";
-    QTest::newRow("file+dir") << "fd" << true << true << true << false << "" << "";
-    QTest::newRow("two dirs") << "dd" << true << true << true << true << "inode/directory" << "inode";
-    QTest::newRow("dir+root dir") << "d/" << true << false << true << true << "inode/directory" << "inode";
-    QTest::newRow("two (text+html) files") << "ff" << true << true << true << false << "" << "text";
-    QTest::newRow("three (text+html+empty) files") << "fff" << true << true << true << false << "" << "";
+    QTest::newRow("one file") << "f" << true << true << true << false << true << "text/plain" << "text";
+    QTest::newRow("one dir") << "d" << true << true << true << true << false << "inode/directory" << "inode";
+    QTest::newRow("root dir") << "/" << true << false << true << true << false << "inode/directory" << "inode";
+    QTest::newRow("file+dir") << "fd" << true << true << true << false << false << "" << "";
+    QTest::newRow("two dirs") << "dd" << true << true << true << true << false << "inode/directory" << "inode";
+    QTest::newRow("dir+root dir") << "d/" << true << false << true << true << false << "inode/directory" << "inode";
+    QTest::newRow("two (text+html) files") << "ff" << true << true << true << false << true << "" << "text";
+    QTest::newRow("three (text+html+empty) files") << "fff" << true << true << true << false << true << "" << "";
     QTest::newRow("http url") << "h" << true << true /*says kio_http...*/
-                              << false << false << "application/octet-stream" << "application";
+                              << false << false << true << "application/octet-stream" << "application";
     QTest::newRow("2 http urls") << "hh" << true << true /*says kio_http...*/
-                                 << false << false << "application/octet-stream" << "application";
+                                 << false << false << true << "application/octet-stream" << "application";
 }
 
 void KFileItemTest::testListProperties()
@@ -472,6 +517,7 @@ void KFileItemTest::testListProperties()
     QFETCH(bool, expectedDeleting);
     QFETCH(bool, expectedIsLocal);
     QFETCH(bool, expectedIsDirectory);
+    QFETCH(bool, expectedIsFile);
     QFETCH(QString, expectedMimeType);
     QFETCH(QString, expectedMimeGroup);
 
@@ -512,6 +558,7 @@ void KFileItemTest::testListProperties()
     QCOMPARE(props.supportsDeleting(), expectedDeleting);
     QCOMPARE(props.isLocal(), expectedIsLocal);
     QCOMPARE(props.isDirectory(), expectedIsDirectory);
+    QCOMPARE(props.isFile(), expectedIsFile);
     QCOMPARE(props.mimeType(), expectedMimeType);
     QCOMPARE(props.mimeGroup(), expectedMimeGroup);
 }
@@ -532,6 +579,10 @@ void KFileItemTest::testIconNameForUrl_data()
         QTest::newRow("videos") << QUrl::fromLocalFile(moviesPath) << (moviesPath == QDir::homePath() ? "user-home" : "folder-videos");
     }
 
+    QTest::newRow("empty") << QUrl() << "unknown";
+    QTest::newRow("relative") << QUrl("foo") << "unknown";
+    QTest::newRow("tilde") << QUrl("~") << "unknown";
+
     // TODO more tests
 }
 
@@ -549,8 +600,8 @@ void KFileItemTest::testIconNameForUrl()
 void KFileItemTest::testMimetypeForRemoteFolder()
 {
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     QUrl url(QStringLiteral("smb://remoteFolder/foo"));
     KFileItem fileItem(entry, url);
 
@@ -563,9 +614,9 @@ void KFileItemTest::testMimetypeForRemoteFolderWithFileType()
     QVERIFY2(QMimeDatabase().mimeTypeForName(udsMimeType).isValid(),
              qPrintable(QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation).join(':'))); // kcoreaddons installed? XDG_DATA_DIRS set?
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-    entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, udsMimeType);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, udsMimeType);
 
     QUrl url(QStringLiteral("smb://remoteFolder/foo"));
     KFileItem fileItem(entry, url);
@@ -576,8 +627,8 @@ void KFileItemTest::testMimetypeForRemoteFolderWithFileType()
 void KFileItemTest::testCurrentMimetypeForRemoteFolder()
 {
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     QUrl url(QStringLiteral("smb://remoteFolder/foo"));
     KFileItem fileItem(entry, url);
 
@@ -588,9 +639,9 @@ void KFileItemTest::testCurrentMimetypeForRemoteFolderWithFileType()
 {
     QString udsMimeType = QStringLiteral("application/x-smb-workgroup");
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
-    entry.insert(KIO::UDSEntry::UDS_MIME_TYPE, udsMimeType);
+    entry.fastInsert(KIO::UDSEntry::UDS_NAME, QStringLiteral("foo"));
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_MIME_TYPE, udsMimeType);
 
     QUrl url(QStringLiteral("smb://remoteFolder/foo"));
     KFileItem fileItem(entry, url);
@@ -611,7 +662,7 @@ void KFileItemTest::testIconNameForCustomFolderIcons()
     cfg.sync();
 
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     KFileItem fileItem(entry, url);
 
     QCOMPARE(fileItem.iconName(), iconName);
@@ -624,7 +675,7 @@ void KFileItemTest::testIconNameForStandardPath()
     QStandardPaths::setTestModeEnabled(true);
 
     KIO::UDSEntry entry;
-    entry.insert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
+    entry.fastInsert(KIO::UDSEntry::UDS_FILE_TYPE, S_IFDIR);
     KFileItem fileItem(entry, url);
 
     QCOMPARE(fileItem.iconName(), iconName);
@@ -657,4 +708,48 @@ void KFileItemTest::testIsReadable()
     QCOMPARE(fileItem.isReadable(), readable);
 }
 
+// Restore permissions so that the QTemporaryDir cleanup can happen (taken from tst_qsavefile.cpp)
+class PermissionRestorer
+{
+    Q_DISABLE_COPY(PermissionRestorer)
+public:
+    explicit PermissionRestorer(const QString& path) : m_path(path) {}
+    ~PermissionRestorer()  { restore(); }
+
+    inline void restore()
+    {
+        QFile file(m_path);
+#ifdef Q_OS_UNIX
+        file.setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
+#else
+        file.setPermissions(QFile::WriteOwner);
+        file.remove();
 #endif
+    }
+private:
+    const QString m_path;
+};
+
+void KFileItemTest::testNonWritableDirectory()
+{
+    // Given a directory with a file in it
+    QTemporaryDir dir;
+    QVERIFY2(dir.isValid(), qPrintable(dir.errorString()));
+    QFile file(dir.path() + "/file1");
+    QVERIFY(file.open(QIODevice::WriteOnly));
+    QCOMPARE(file.write("Hello"), Q_INT64_C(5));
+    file.close();
+    // ... which is then made non-writable
+    QVERIFY(QFile(dir.path()).setPermissions(QFile::ReadOwner | QFile::ExeOwner));
+    PermissionRestorer permissionRestorer(dir.path());
+
+    // When using KFileItemListProperties on the file
+    const KFileItem item(QUrl::fromLocalFile(file.fileName()));
+    KFileItemListProperties props(KFileItemList() << item);
+
+    // Then it should say moving is not supported
+    QVERIFY(!props.supportsMoving());
+    QVERIFY(props.supportsWriting()); // but we can write to the file itself
+}
+
+#endif // Q_OS_WIN
