@@ -19,7 +19,7 @@
 #ifndef KFILEITEMTEST_H
 #define KFILEITEMTEST_H
 
-#include <QtCore/QObject>
+#include <QObject>
 
 class KFileItemTest : public QObject
 {
@@ -36,6 +36,7 @@ private Q_SLOTS:
     void testHiddenFile();
     void testMimeTypeOnDemand();
     void testCmp();
+    void testCmpByUrl();
     void testRename();
     void testRefresh();
     void testDotDirectory();
@@ -59,6 +60,9 @@ private Q_SLOTS:
     // KFileItemListProperties tests
     void testListProperties_data();
     void testListProperties();
+#ifndef Q_OS_WIN
+    void testNonWritableDirectory();
+#endif
 
     // KIO global tests
     void testIconNameForUrl_data();
