@@ -63,7 +63,7 @@
 
 static QString expandTilde(const QString &name, bool isfile = false)
 {
-    if (!name.isEmpty() && (!isfile || name[0] == '\\')) {
+    if (!name.isEmpty() && (!isfile || name[0] == QLatin1Char('\\'))) {
         const QString expandedName = KShell::tildeExpand(name);
         // When a tilde mark cannot be properly expanded, the above call
         // returns an empty string...
@@ -167,12 +167,12 @@ void KNewFileMenuSingleton::parseFiles()
                 QString type = desktopFile.readType();
                 if (type == QLatin1String("Link")) {
                     templatePath = desktopFile.desktopGroup().readPathEntry("URL", QString());
-                    if (templatePath[0] != '/' && !templatePath.startsWith(QLatin1String("__"))) {
+                    if (templatePath[0] != QLatin1Char('/') && !templatePath.startsWith(QLatin1String("__"))) {
                         if (templatePath.startsWith(QLatin1String("file:/"))) {
                             templatePath = QUrl(templatePath).toLocalFile();
                         } else {
                             // A relative path, then (that's the default in the files we ship)
-                            QString linkDir = filePath.left(filePath.lastIndexOf('/') + 1 /*keep / */);
+                            const QStringRef linkDir = filePath.leftRef(filePath.lastIndexOf(QLatin1Char('/')) + 1 /*keep / */);
                             //qDebug() << "linkDir=" << linkDir;
                             templatePath = linkDir + templatePath;
                         }
@@ -191,7 +191,7 @@ void KNewFileMenuSingleton::parseFiles()
             if (text.isEmpty()) {
                 text = QUrl(filePath).fileName();
                 if (text.endsWith(QLatin1String(".desktop"))) {
-                    text.truncate(text.length() - 8);
+                    text.chop(8);
                 }
             }
             templ.text = text;
@@ -238,10 +238,11 @@ public:
 class KNewFileMenuPrivate
 {
 public:
-    KNewFileMenuPrivate(KNewFileMenu *qq)
+    explicit KNewFileMenuPrivate(KNewFileMenu *qq)
         : m_menuItemsVersion(0),
           m_modal(true),
           m_viewShowsHiddenFiles(false),
+          m_firstFileEntry(nullptr),
           q(qq)
     {}
 
@@ -372,7 +373,9 @@ public:
     QString m_text;
     bool m_viewShowsHiddenFiles;
 
-    KNewFileMenu *q;
+    KNewFileMenuSingleton::Entry *m_firstFileEntry;
+
+    KNewFileMenu * const q;
 
     KNewFileMenuCopyData m_copyData;
 };
@@ -393,8 +396,7 @@ bool KNewFileMenuPrivate::checkSourceExists(const QString &src)
 
         KMessageBox::createKMessageBox(dialog, buttonBox, QMessageBox::Warning,
                                        i18n("<qt>The template file <b>%1</b> does not exist.</qt>", src),
-                                       QStringList(), QString(), nullptr, KMessageBox::NoExec,
-                                       QString());
+                                       QStringList(), QString(), nullptr, KMessageBox::NoExec);
 
         dialog->show();
 
@@ -431,11 +433,10 @@ void KNewFileMenuPrivate::confirmCreatingHiddenDir(const QString &name)
                                    QStringList(),
                                    i18n("Do not ask again"),
                                    nullptr,
-                                   KMessageBox::NoExec,
-                                   QString());
+                                   KMessageBox::NoExec);
 
     QObject::connect(buttonBox, SIGNAL(accepted()), q, SLOT(_k_slotCreateHiddenDirectory()));
-    QObject::connect(buttonBox, SIGNAL(rejected()), q, SLOT(createDirectory()));
+    QObject::connect(buttonBox, &QDialogButtonBox::rejected, q, &KNewFileMenu::createDirectory);
 
     m_fileDialog = confirmDialog;
     confirmDialog->show();
@@ -459,7 +460,7 @@ void KNewFileMenuPrivate::executeOtherDesktopFile(const KNewFileMenuSingleton::E
         text.append(QStringLiteral(".desktop"));
 
         const QUrl directory = mostLocalUrl(*it);
-        const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + '/' + KIO::encodeFileName(text));
+        const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + QLatin1Char('/') + KIO::encodeFileName(text));
         if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
             text = KIO::suggestName(directory, text);
         }
@@ -503,7 +504,7 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
     m_copyData.m_src = entry.templatePath;
 
     const QUrl directory = mostLocalUrl(m_popupFiles.first());
-    const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + '/' + KIO::encodeFileName(text));
+    const QUrl defaultFile = QUrl::fromLocalFile(directory.toLocalFile() + QLatin1Char('/') + KIO::encodeFileName(text));
     if (defaultFile.isLocalFile() && QFile::exists(defaultFile.toLocalFile())) {
         text = KIO::suggestName(directory, text);
     }
@@ -524,8 +525,8 @@ void KNewFileMenuPrivate::executeRealFileOrDir(const KNewFileMenuSingleton::Entr
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(fileDialog);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    QObject::connect(buttonBox, SIGNAL(accepted()), fileDialog, SLOT(accept()));
-    QObject::connect(buttonBox, SIGNAL(rejected()), fileDialog, SLOT(reject()));
+    QObject::connect(buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
+    QObject::connect(buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
 
     layout->addWidget(label);
     layout->addWidget(lineEdit);
@@ -617,7 +618,7 @@ void KNewFileMenuPrivate::executeStrategy()
             KIO::FileUndoManager::self()->recordCopyJob(job);
         }
         KJobWidgets::setWindow(kjob, m_parentWidget);
-        QObject::connect(kjob, SIGNAL(result(KJob*)), q, SLOT(slotResult(KJob*)));
+        QObject::connect(kjob, &KJob::result, q, &KNewFileMenu::slotResult);
     }
 }
 
@@ -689,15 +690,10 @@ void KNewFileMenuPrivate::fillMenu()
                     }
 
                     menu->addAction(act);
-
-                    QAction *sep = new QAction(q);
-                    sep->setSeparator(true);
-                    menu->addAction(sep);
+                    menu->addSeparator();
                 } else {
                     if (lastTemplatePath.startsWith(QDir::homePath()) && !templatePath.startsWith(QDir::homePath())) {
-                        QAction *sep = new QAction(q);
-                        sep->setSeparator(true);
-                        menu->addAction(sep);
+                        menu->addSeparator();
                     }
                     if (!m_supportedMimeTypes.isEmpty()) {
                         bool keep = false;
@@ -754,6 +750,20 @@ void KNewFileMenuPrivate::fillMenu()
                             menu->addAction(act);
                         }
                     } else {
+                        if (!m_firstFileEntry) {
+                            m_firstFileEntry = &entry;
+                            // If there is a shortcut available in the action collection, use it.
+                            QAction *act2 = m_actionCollection->action(QStringLiteral("create_file"));
+                            if (act2) {
+                                act->setShortcuts(act2->shortcuts());
+                                // Both actions have now the same shortcut, so this will prevent the "Ambiguous shortcut detected" dialog.
+                                act->setShortcutContext(Qt::WidgetShortcut);
+                                // We also need to react to shortcut changes.
+                                QObject::connect(act2, &QAction::changed, act, [=]() {
+                                    act->setShortcuts(act2->shortcuts());
+                                });
+                            }
+                        }
                         menu->addAction(act);
                     }
                 }
@@ -761,17 +771,12 @@ void KNewFileMenuPrivate::fillMenu()
             lastTemplatePath = entry.templatePath;
         } else { // Separate system from personal templates
             Q_ASSERT(entry.entryType != 0);
-
-            QAction *sep = new QAction(q);
-            sep->setSeparator(true);
-            menu->addAction(sep);
+            menu->addSeparator();
         }
     }
 
     if (m_supportedMimeTypes.isEmpty()) {
-        QAction *sep = new QAction(q);
-        sep->setSeparator(true);
-        menu->addAction(sep);
+        menu->addSeparator();
         if (linkURL) {
             menu->addAction(linkURL);
         }
@@ -872,20 +877,20 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
                 KGuiItem::assign(buttonBox->button(QDialogButtonBox::Ok), enterNewNameGuiItem);
 
                 KMessageBox::createKMessageBox(confirmDialog, buttonBox, QMessageBox::Critical,
-                                   xi18n("Could not create a folder with the name <filename>%1</filename><nl/>because it is reserved for use by the operating system.", name),
+                                   xi18nc("@info", "Could not create a folder with the name <filename>%1</filename><nl/>because it is reserved for use by the operating system.", name),
                                    QStringList(),
                                    QString(),
                                    nullptr,
                                    KMessageBox::NoExec,
                                    QString());
 
-                QObject::connect(buttonBox, SIGNAL(accepted()), q, SLOT(createDirectory()));
+                QObject::connect(buttonBox, &QDialogButtonBox::accepted, q, &KNewFileMenu::createDirectory);
                 m_fileDialog = confirmDialog;
                 confirmDialog->show();
                 _k_slotAbortDialog();
                 return;
             }
-            if (!m_viewShowsHiddenFiles && name.startsWith('.')) {
+            if (!m_viewShowsHiddenFiles && name.startsWith(QLatin1Char('.'))) {
                 if (!writeHiddenDir) {
                     confirmCreatingHiddenDir(name);
                     return;
@@ -896,18 +901,24 @@ void KNewFileMenuPrivate::_k_slotCreateDirectory(bool writeHiddenDir)
         }
     }
 
-    // Note that we use mkpath so that a/b/c works.
-    // On the other hand it means that passing the name of a directory that already exists will do nothing.
-    KIO::Job *job = KIO::mkpath(url, baseUrl);
-    job->setProperty("mkpathUrl", url);
-    KJobWidgets::setWindow(job, m_parentWidget);
+    KIO::Job *job;
+    if (name.contains(QLatin1Char('/'))) {
+        // If the name contains any slashes, use mkpath so that a/b/c works.
+        job = KIO::mkpath(url, baseUrl);
+        KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkpath, QList<QUrl>(), url, job);
+    } else {
+        // If not, use mkdir so it will fail if the name of an existing folder was used
+        job = KIO::mkdir(url);
+        KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkdir, QList<QUrl>(), url, job);
+    }
+    job->setProperty("newDirectoryURL", url);
     job->uiDelegate()->setAutoErrorHandlingEnabled(true);
-    KIO::FileUndoManager::self()->recordJob(KIO::FileUndoManager::Mkpath, QList<QUrl>(), url, job);
+    KJobWidgets::setWindow(job, m_parentWidget);
 
     if (job) {
         // We want the error handling to be done by slotResult so that subclasses can reimplement it
         job->uiDelegate()->setAutoErrorHandlingEnabled(false);
-        QObject::connect(job, SIGNAL(result(KJob*)), q, SLOT(slotResult(KJob*)));
+        QObject::connect(job, &KJob::result, q, &KNewFileMenu::slotResult);
     }
     _k_slotAbortDialog();
 }
@@ -933,13 +944,13 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     #ifdef Q_OS_UNIX
         QString xdgUserDirs = QStandardPaths::locate(QStandardPaths::ConfigLocation, QStringLiteral("user-dirs.dirs"), QStandardPaths::LocateFile);
         QFile xdgUserDirsFile(xdgUserDirs);
-        if (xdgUserDirsFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
+        if (!xdgUserDirs.isEmpty() && xdgUserDirsFile.open(QIODevice::ReadOnly | QIODevice::Text)) {
             QTextStream in(&xdgUserDirsFile);
             while (!in.atEnd()) {
                 QString line = in.readLine();
-                if (line.startsWith("XDG_TEMPLATES_DIR=")) {
+                if (line.startsWith(QLatin1String("XDG_TEMPLATES_DIR="))) {
                     QString xdgTemplates = line.mid(19, line.size()-20);
-                    xdgTemplates.replace(QString("$HOME"), QDir::homePath());
+                    xdgTemplates.replace(QStringLiteral("$HOME"), QDir::homePath());
                     QDir xdgTemplatesDir(xdgTemplates);
                     if (xdgTemplatesDir.exists()) {
                         installedTemplates << xdgTemplates;
@@ -955,7 +966,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     // Ensure any changes in the templates dir will call this
     if (! s->dirWatch) {
         s->dirWatch = new KDirWatch;
-        for (const QString &dir : installedTemplates) {
+        for (const QString &dir : qAsConst(installedTemplates)) {
             s->dirWatch->addDir(dir);
         }
         QObject::connect(s->dirWatch, SIGNAL(dirty(QString)),
@@ -978,6 +989,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     for (const QString &path : templates) {
         dir.setPath(path);
         const QStringList &entryList(dir.entryList(QStringList() << QStringLiteral("*.desktop"), QDir::Files));
+        files.reserve(files.size() + entryList.size());
         Q_FOREACH (const QString &entry, entryList) {
             const QString file = concatPaths(dir.path(), entry);
             files.append(file);
@@ -988,7 +1000,7 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
     QMap<QString, EntryWithName> ulist; // entries with unique URLs
     Q_FOREACH (const QString &file, files) {
         //qDebug() << file;
-        if (file[0] != '.') {
+        if (file[0] != QLatin1Char('.')) {
             KNewFileMenuSingleton::Entry e;
             e.filePath = file;
             e.entryType = KNewFileMenuSingleton::Unknown; // not parsed yet
@@ -1001,13 +1013,13 @@ void KNewFileMenuPrivate::_k_slotFillTemplates()
             QString url = config.desktopGroup().readEntry("URL");
             QString key = config.desktopGroup().readEntry("Name");
             if (file.endsWith(QLatin1String("Directory.desktop"))) {
-                key.prepend('0');
+                key.prepend(QLatin1Char('0'));
             } else if (file.startsWith(QDir::homePath())) {
-                key.prepend('1');
+                key.prepend(QLatin1Char('1'));
             } else if (file.endsWith(QLatin1String("TextFile.desktop"))) {
-                key.prepend('2');
+                key.prepend(QLatin1Char('2'));
             } else {
-                key.prepend('3');
+                key.prepend(QLatin1Char('3'));
             }
             EntryWithName en = { key, e };
             if (ulist.contains(url)) {
@@ -1197,7 +1209,7 @@ void KNewFileMenu::createDirectory()
     QString name = d->m_text.isEmpty() ? i18nc("Default name for a new folder", "New Folder") :
                    d->m_text;
 
-    if (baseUrl.isLocalFile() && QFileInfo(baseUrl.toLocalFile() + '/' + name).exists()) {
+    if (baseUrl.isLocalFile() && QFileInfo::exists(baseUrl.toLocalFile() + QLatin1Char('/') + name)) {
         name = KIO::suggestName(baseUrl, name);
     }
 
@@ -1218,8 +1230,8 @@ void KNewFileMenu::createDirectory()
 
     QDialogButtonBox *buttonBox = new QDialogButtonBox(fileDialog);
     buttonBox->setStandardButtons(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
-    QObject::connect(buttonBox, SIGNAL(accepted()), fileDialog, SLOT(accept()));
-    QObject::connect(buttonBox, SIGNAL(rejected()), fileDialog, SLOT(reject()));
+    QObject::connect(buttonBox, &QDialogButtonBox::accepted, fileDialog, &QDialog::accept);
+    QObject::connect(buttonBox, &QDialogButtonBox::rejected, fileDialog, &QDialog::reject);
 
     layout->addWidget(label);
     layout->addWidget(lineEdit);
@@ -1236,6 +1248,20 @@ void KNewFileMenu::createDirectory()
     lineEdit->setFocus();
 }
 
+void KNewFileMenu::createFile()
+{
+    if (d->m_popupFiles.isEmpty()) {
+        return;
+    }
+
+    checkUpToDate();
+    if (!d->m_firstFileEntry) {
+        return;
+    }
+
+    d->executeRealFileOrDir(*d->m_firstFileEntry);
+}
+
 bool KNewFileMenu::isModal() const
 {
     return d->m_modal;
@@ -1290,26 +1316,23 @@ void KNewFileMenu::slotResult(KJob *job)
         static_cast<KIO::Job *>(job)->uiDelegate()->showErrorMessage();
     } else {
         // Was this a copy or a mkdir?
-        KIO::CopyJob *copyJob = ::qobject_cast<KIO::CopyJob *>(job);
-        if (copyJob) {
-            const QUrl destUrl = copyJob->destUrl();
-            const QUrl localUrl = d->mostLocalUrl(destUrl);
-            if (localUrl.isLocalFile()) {
-                // Normal (local) file. Need to "touch" it, kio_file copied the mtime.
-                (void) ::utime(QFile::encodeName(localUrl.toLocalFile()).constData(), nullptr);
-            }
-            emit fileCreated(destUrl);
-        } else if (KIO::SimpleJob *simpleJob = ::qobject_cast<KIO::SimpleJob *>(job)) {
-            // Called in the storedPut() case
-            org::kde::KDirNotify::emitFilesAdded(simpleJob->url().adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
-            emit fileCreated(simpleJob->url());
-        }  else {
-            // Can be mkdir
-            QUrl mkpathUrl = job->property("mkpathUrl").toUrl();
-            if (mkpathUrl.isValid()) {
-                emit directoryCreated(mkpathUrl);
-            } else {
-                qWarning() << "Neither copy, put nor mkdir, internal error";
+        if (job->property("newDirectoryURL").isValid()) {
+            QUrl newDirectoryURL = job->property("newDirectoryURL").toUrl();
+            emit directoryCreated(newDirectoryURL);
+        } else {
+            KIO::CopyJob *copyJob = ::qobject_cast<KIO::CopyJob *>(job);
+            if (copyJob) {
+                const QUrl destUrl = copyJob->destUrl();
+                const QUrl localUrl = d->mostLocalUrl(destUrl);
+                if (localUrl.isLocalFile()) {
+                    // Normal (local) file. Need to "touch" it, kio_file copied the mtime.
+                    (void) ::utime(QFile::encodeName(localUrl.toLocalFile()).constData(), nullptr);
+                }
+                emit fileCreated(destUrl);
+            } else if (KIO::SimpleJob *simpleJob = ::qobject_cast<KIO::SimpleJob *>(job)) {
+                // Called in the storedPut() case
+                org::kde::KDirNotify::emitFilesAdded(simpleJob->url().adjusted(QUrl::RemoveFilename | QUrl::StripTrailingSlash));
+                emit fileCreated(simpleJob->url());
             }
         }
     }
