@@ -28,8 +28,8 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 #define SAVE_DELAY 3 // Save after 3 minutes
 
-#include <QtCore/QTimer>
-#include <QtCore/QFile>
+#include <QTimer>
+#include <QFile>
 #include <QDBusConnection>
 
 #include <kconfig.h>
@@ -112,13 +112,13 @@ KCookieServer::KCookieServer(QObject *parent, const QList<QVariant> &)
     mAdvicePending = false;
     mTimer = new QTimer();
     mTimer->setSingleShot(true);
-    connect(mTimer, SIGNAL(timeout()), SLOT(slotSave()));
+    connect(mTimer, &QTimer::timeout, this, &KCookieServer::slotSave);
     mConfig = new KConfig(QStringLiteral("kcookiejarrc"));
     mCookieJar->loadConfig(mConfig);
     mFilename = getOrCreateCookieJarDir().absoluteFilePath(QStringLiteral("cookies"));
     mCookieJar->loadCookies(mFilename);
-    connect(this, SIGNAL(windowUnregistered(qlonglong)),
-            this, SLOT(slotDeleteSessionCookies(qlonglong)));
+    connect(this, &KDEDModule::windowUnregistered,
+            this, &KCookieServer::slotDeleteSessionCookies);
 }
 
 KCookieServer::~KCookieServer()
@@ -393,7 +393,7 @@ KCookieServer::findDomains()
     Q_FOREACH (const QString &domain, mCookieJar->getDomainList()) {
         // Ignore domains that have policy set for but contain
         // no cookies whatsoever...
-        const KHttpCookieList *list =  mCookieJar->getCookieList(domain, QLatin1String(""));
+        const KHttpCookieList *list =  mCookieJar->getCookieList(domain, QString());
         if (list && !list->isEmpty()) {
             result << domain;
         }
@@ -573,7 +573,7 @@ KCookieServer::getDomainAdvice(const QString &url)
             // that we only want to check "domain" if it matches the
             // fqdn of the requested URL.
             const QString &domain = it.next();
-            if (domain.at(0) == '.' || domain == fqdn) {
+            if (domain.at(0) == QLatin1Char('.') || domain == fqdn) {
                 advice = mCookieJar->getDomainAdvice(domain);
             }
         }
