@@ -27,8 +27,7 @@
 #include <kurifilter.h>
 #include <KAboutData>
 #include <KLocalizedString>
-#include <KService>
-#include <KServiceTypeTrader>
+#include <KPluginMetaData>
 #include <KPluginFactory>
 #include <KPluginLoader>
 
@@ -36,9 +35,11 @@ K_PLUGIN_FACTORY(KURIFilterModuleFactory, registerPlugin<KURIFilterModule>();)
 
 KURIFilterModule::KURIFilterModule(QWidget *parent, const QVariantList &args)
     : KCModule(parent, args),
-      m_widget(0)
+      m_widget(nullptr)
 {
-    KAboutData *about = new KAboutData(QStringLiteral("kcm_webshortcuts"), QString(), i18n("Web Shortcuts"), QStringLiteral("0.1"), KAboutLicense::GPL);
+    KAboutData *about = new KAboutData(QStringLiteral("kcm_webshortcuts"), i18n("Web Shortcuts"),
+                                       QStringLiteral("0.1"), i18n("Configure enhanced browsing features"),
+                                       KAboutLicense::GPL);
     setAboutData(about);
 
     KCModule::setButtons(KCModule::Buttons(KCModule::Default | KCModule::Apply | KCModule::Help));
@@ -56,17 +57,20 @@ KURIFilterModule::KURIFilterModule(QWidget *parent, const QVariantList &args)
 
     QMap<QString, KCModule *> helper;
     // Load the plugins. This saves a public method in KUriFilter just for this.
-    const KService::List offers = KServiceTypeTrader::self()->query("KUriFilter/Plugin");
-    KService::List::ConstIterator it = offers.begin();
-    const KService::List::ConstIterator end = offers.end();
-    for (; it != end; ++it) {
-        KUriFilterPlugin *plugin = (*it)->createInstance<KUriFilterPlugin>(this);
-        if (plugin) {
-            KCModule *module = plugin->configModule(this, 0);
-            if (module) {
-                modules.append(module);
-                helper.insert(plugin->configName(), module);
-                connect(module, SIGNAL(changed(bool)), SIGNAL(changed(bool)));
+
+    QVector<KPluginMetaData> plugins = KPluginLoader::findPlugins(QStringLiteral("kf5/urifilters"));
+    for (const KPluginMetaData &pluginMetaData : plugins) {
+        KPluginFactory *factory = qobject_cast<KPluginFactory *>(pluginMetaData.instantiate());
+        if (factory) {
+            KUriFilterPlugin *plugin = factory->create<KUriFilterPlugin>(nullptr);
+            if (plugin) {
+                KCModule *module = plugin->configModule(this, nullptr);
+                if (module) {
+                    modules.append(module);
+                    helper.insert(plugin->configName(), module);
+                    connect(module, QOverload<bool>::of(&KCModule::changed),
+                            this, QOverload<bool>::of(&KCModule::changed));
+                }
             }
         }
     }
