@@ -20,21 +20,21 @@
 
 #include "script.h"
 
-#include <QtCore/QString>
-#include <QtCore/QRegExp>
-#include <QtCore/QDateTime>
-#include <QtCore/QTimer>
-#include <QtCore/QEventLoop>
+#include <QString>
+#include <QRegExp>
+#include <QDateTime>
+#include <QTimer>
+#include <QEventLoop>
 #include <QUrl>
 
-#include <QtNetwork/QHostInfo>
-#include <QtNetwork/QHostAddress>
-#include <QtNetwork/QNetworkInterface>
+#include <QHostInfo>
+#include <QHostAddress>
+#include <QNetworkInterface>
 
-#include <QtScript/QScriptValue>
-#include <QtScript/QScriptEngine>
-#include <QtScript/QScriptProgram>
-#include <QtScript/QScriptContextInfo>
+#include <QScriptValue>
+#include <QScriptEngine>
+#include <QScriptProgram>
+#include <QScriptContextInfo>
 
 #include <klocalizedstring.h>
 #include <kio/hostinfo.h>
@@ -46,7 +46,6 @@ namespace
 static int findString(const QString &s, const char *const *values)
 {
     int index = 0;
-    const QString lower = s.toLower();
     for (const char *const *p = values; *p; ++p, ++index) {
         if (s.compare(QLatin1String(*p), Qt::CaseInsensitive) == 0) {
             return index;
@@ -271,9 +270,7 @@ QScriptValue IsInNet(QScriptContext *context, QScriptEngine *engine)
     try {
         const Address info = Address::resolve(context->argument(0).toString());
         bool isInSubNet = false;
-        QString subnetStr = context->argument(1).toString();
-        subnetStr += QLatin1Char('/');
-        subnetStr += context->argument(2).toString();
+        const QString subnetStr = context->argument(1).toString() + QLatin1Char('/') + context->argument(2).toString();
         const QPair<QHostAddress, int> subnet = QHostAddress::parseSubnet(subnetStr);
         Q_FOREACH (const QHostAddress &address, info.addresses()) {
             if (!isSpecialAddress(address) && isIPv4Address(address) && address.isInSubnet(subnet)) {
@@ -607,7 +604,7 @@ QScriptValue DNSResolveEx(QScriptContext *context, QScriptEngine *engine)
             }
         }
         if (!addressList.isEmpty()) {
-            resolvedAddress = addressList.join(QStringLiteral(";"));
+            resolvedAddress = addressList.join(QLatin1Char(';'));
         }
 
         return engine->toScriptValue(resolvedAddress);
@@ -633,7 +630,7 @@ QScriptValue MyIpAddressEx(QScriptContext *context, QScriptEngine *engine)
         }
     }
 
-    return engine->toScriptValue(ipAddressList.join(QStringLiteral(";")));
+    return engine->toScriptValue(ipAddressList.join(QLatin1Char(';')));
 }
 
 // sortIpAddressList(ipAddressList)
