@@ -38,6 +38,7 @@
 
 #include <QDateTime>
 #include <QDBusConnection>
+#include <QFileInfo>
 
 #include <assert.h>
 
@@ -64,7 +65,7 @@ static QDataStream &operator>>(QDataStream &stream, BasicOperation &op)
     stream >> op.m_valid >> type >> op.m_renamed
            >> op.m_src >> op.m_dst >> op.m_target >> mtime;
     op.m_type = static_cast<BasicOperation::Type>(type);
-    op.m_mtime = QDateTime::fromMSecsSinceEpoch(1000 * mtime);
+    op.m_mtime = QDateTime::fromMSecsSinceEpoch(1000 * mtime, Qt::UTC);
     return stream;
 }
 
@@ -158,13 +159,13 @@ CommandRecorder::CommandRecorder(FileUndoManager::CommandType op, const QList<QU
     m_cmd.m_serialNumber = FileUndoManager::self()->newCommandSerialNumber();
     m_cmd.m_src = src;
     m_cmd.m_dst = dst;
-    connect(job, SIGNAL(result(KJob*)),
-            this, SLOT(slotResult(KJob*)));
-    if (qobject_cast<KIO::CopyJob *>(job)) {
-        connect(job, SIGNAL(copyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)),
-                this, SLOT(slotCopyingDone(KIO::Job*,QUrl,QUrl,QDateTime,bool,bool)));
-        connect(job, SIGNAL(copyingLinkDone(KIO::Job*,QUrl,QString,QUrl)),
-                this, SLOT(slotCopyingLinkDone(KIO::Job*,QUrl,QString,QUrl)));
+    connect(job, &KJob::result,
+            this, &CommandRecorder::slotResult);
+    if (auto *copyJob = qobject_cast<KIO::CopyJob*>(job)) {
+        connect(copyJob, &KIO::CopyJob::copyingDone,
+                this, &CommandRecorder::slotCopyingDone);
+        connect(copyJob, &KIO::CopyJob::copyingLinkDone,
+                this, &CommandRecorder::slotCopyingLinkDone);
     } else if (KIO::MkpathJob *mkpathJob = qobject_cast<KIO::MkpathJob *>(job)) {
         connect(mkpathJob, &KIO::MkpathJob::directoryCreated,
                 this, &CommandRecorder::slotDirectoryCreated);
@@ -382,10 +383,14 @@ void FileUndoManager::undo()
     BasicOperation::Stack::Iterator it = opStack.begin();
     for (; it != opStack.end(); ++it) {
         BasicOperation::Type type = (*it).m_type;
+        const auto destination = (*it).m_dst;
         if (type == BasicOperation::File && commandType == FileUndoManager::Copy) {
-            itemsToDelete.append((*it).m_dst);
+            if (destination.isLocalFile() && !QFileInfo::exists(destination.toLocalFile())) {
+                continue;
+            }
+            itemsToDelete.append(destination);
         } else if (commandType == FileUndoManager::Mkpath) {
-            itemsToDelete.append((*it).m_dst);
+            itemsToDelete.append(destination);
         }
     }
     if (commandType == FileUndoManager::Mkdir || commandType == FileUndoManager::Put) {
@@ -395,6 +400,9 @@ void FileUndoManager::undo()
         if (!d->m_uiInterface->confirmDeletion(itemsToDelete)) {
             return;
         }
+    } else if (commandType == FileUndoManager::Copy) {
+        d->slotPop();
+        return;
     }
 
     d->slotPop();
@@ -476,7 +484,7 @@ void FileUndoManagerPrivate::slotResult(KJob *job)
         BasicOperation op = m_current.m_opStack.last();
         //qDebug() << "stat result for " << op.m_dst;
         KIO::StatJob *statJob = static_cast<KIO::StatJob *>(job);
-        const QDateTime mtime = QDateTime::fromMSecsSinceEpoch(1000 * statJob->statResult().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1));
+        const QDateTime mtime = QDateTime::fromMSecsSinceEpoch(1000 * statJob->statResult().numberValue(KIO::UDSEntry::UDS_MODIFICATION_TIME, -1), Qt::UTC);
         if (mtime != op.m_mtime) {
             //qDebug() << op.m_dst << " was modified after being copied!";
             QDateTime srcTime = op.m_mtime.toLocalTime();
@@ -521,8 +529,8 @@ void FileUndoManagerPrivate::undoStep()
         if (m_uiInterface) {
             KJobWidgets::setWindow(m_currentJob, m_uiInterface->parentWidget());
         }
-        QObject::connect(m_currentJob, SIGNAL(result(KJob*)),
-                         this, SLOT(slotResult(KJob*)));
+        QObject::connect(m_currentJob, &KJob::result,
+                         this, &FileUndoManagerPrivate::slotResult);
     }
 }
 
